#ifndef HARKIO_HARKCONFIG_HARKCONFIGFILETEXTREADER_HPP_
#define HARKIO_HARKCONFIG_HARKCONFIGFILETEXTREADER_HPP_

#include <istream>
#include <string>
#include <boost/shared_ptr.hpp>
#include <harkio2/AnalyzeError.hpp>
#include <harkio2/harkconfig/HarkConfigFileReader.hpp>

/** @file HarkConfigFileTextReader.hpp
 * @brief HarkConfigFileTextReader クラスの定義
 */

namespace harkio2
{
namespace harkconfig
{

class HarkConfigFile;

/** @brief テキスト形式のHARK-configファイルに対応した HarkConfigFileReader の実装。
 */
class HarkConfigFileTextReader :
    public HarkConfigFileReader
{
public:
    /** @brief テキストリーダーを構築する。
     *
     * データ源はstd::istream（のサブクラス）であり、引数'in'で指定される。
     * 入力ストリームはシークをサポートする必要はない。
     *
     * @param in データ読み込み元の入力ストリーム
     */
    HarkConfigFileTextReader(std::istream& in);

    /** @brief HarkConfigFile データ構造をコンストラクタで指定されたストリームから読み込み、返す。
     * @return 読み込んだデータ
     */
    virtual boost::shared_ptr<HarkConfigFile> Read() throw (harkio2::AnalyzeError);

    /** @brief Reads a line from the input stream of this instance.
     *
     * 内部使用向け関数。
     *
     * This method works like std::getline (in <string> header).
     * This method reads a line from a given input stream,
     * and eliminates comments from the line.
     * If a read line is a blank line or consists of only comments,
     * this method reads further lines until this method finds
     * a valid line.
     * @param in A input stream to read
     * @param line A string buffer this method will write the result to.
     * @return Given input stream 'in'
     */
    static std::istream& GetValidLine(std::istream& in, std::string& line);

    /** @brief Reads a line from the input stream of this instance.
     *
     * 内部使用向け関数。
     *
     * This method is the same as GetValidLine(istream, string),
     * except that this method will also return the number of read lines.
     * @param in A input stream to read
     * @param line A string buffer this method will write the result to.
     * @param numReadLine The number of read lines
     * @return Given input stream 'in'
     */
    static std::istream& GetValidLine(std::istream& in, std::string& line, unsigned int& numReadLine);

    /** @brief Eliminates comments from a line.
     *
     * 内部使用向け関数。
     *
     * @param line A line you want to eliminate comments from.
     * @return Comment eliminated line.
     */
    static std::string EliminateComment(const std::string& line);

    /** @brief Shrink a line by eliminating both ends spaces.
     *
     * 内部使用向け関数。
     *
     * @param line A line you want to eliminate both ends spaces
     * @return Space eliminated line.
     */
    static std::string Shrink(const std::string& line);

private:
    /// @brief Input stream from which this instance reads data.
    std::istream& in;

};

}
}

#endif // HARKIO_HARKCONFIG_HARKCONFIGFILETEXTREADER_HPP_

