#include <harkio2/harkconfig/nodeparsers.hpp>

#include <map>
#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>
#include <libxml++/parsers/textreader.h>

#include <harkio2/AnalyzeError.hpp>
#include <harkio2/xmlhelper/xmlhelper.hpp>
#include <harkio2/Position.hpp>
#include <harkio2/FileType.hpp>
#include <harkio2/TypedFilename.hpp>
#include <harkio2/harkconfig/Source.hpp>
#include <harkio2/harkconfig/File.hpp>
#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFile.hpp>
#include <harkio2/harkconfig/TSPListFile.hpp>
#include <harkio2/harkconfig/MicArrayLocationFile.hpp>
#include <harkio2/harkconfig/NoiseLocationFile.hpp>


using namespace std;
using namespace boost;
using namespace Glib;
using namespace xmlpp;
using namespace harkio2::xmlhelper;

namespace harkio2
{

namespace harkconfig
{

bool ConvertFileType(const Glib::ustring& src, harkio2::FileType::Kind& dst)
{
    if (src == "float")
    {
        dst = harkio2::FileType::FLOAT;
    }
    else if (src == "wave")
    {
        dst = harkio2::FileType::WAVE;
    }
    else
    {
        return false;
    }
    return true;
}

template <typename T>
T DegreeToRadian(T deg)
{
    return M_PI * deg / 180.0;
}

template <typename T>
T ParseGeneral(xmlpp::TextReader& reader, const Glib::ustring& elementName)
{
    if (IsElement(reader, elementName))
    {
        return lexical_cast<T>(reader.read_inner_xml());
    }
    else
    {
        throw AnalyzeError((format("ParseGeneral: The given node isn't %s start tag.") % elementName).str(), GetLine(reader));
    }
}

template <>
Glib::ustring ParseGeneral<Glib::ustring>(xmlpp::TextReader& reader, const Glib::ustring& elementName)
{
    if (IsElement(reader, elementName))
    {
        return reader.read_inner_xml();
    }
    else
    {
        throw AnalyzeError((format("ParseGeneral: The given node isn't %s start tag.") % elementName).str(), GetLine(reader));
    }
}

harkio2::Position<float> ParsePosition(TextReader& reader)
{
    if (IsElement(reader, "Position"))
    {
        typedef map<ustring, ustring> attr_table;
        typedef map<ustring, float> value_table;
        attr_table attrs;
        value_table values;

        // attributes (unit, angle, type)
        while (reader.move_to_next_attribute())
        {
            attrs[reader.get_name()] = reader.get_value();
        }

        // data (X, Y, Z, Theta, Phi, R)
        while (reader.read() && !IsEndElement(reader, "Position"))
        {
            if (reader.get_node_type() == TextReader::Element)
            {
                values[reader.get_name()] = lexical_cast<float>(reader.read_inner_xml());
            }
        }

        // error check
        if (Contains(attrs, "type") == false)
        {
            throw AnalyzeError("ParsePosition: 'type' attribute can't be omitted", GetLine(reader));
        }

        // direction
        if (Contains(attrs, "unit") == false || attrs["unit"] == "none")
        {
            if (attrs["type"] == "cartesian" || attrs["type"] == "unitvector")
            {
                return MakeDirectionFromVector<float>(
                        Vector3<float>(values["X"], values["Y"], values["Z"]));
            }
            else if (attrs["type"] == "polar")
            {
                if (Contains(attrs, "angle") == false || attrs["angle"] == "degree")
                {
                    return MakeDirectionFromPolar<float>(
                            DegreeToRadian(values["Theta"]), DegreeToRadian(values["Phi"]));
                }
                else if (attrs["angle"] == "radian")
                {
                    return MakeDirectionFromPolar<float>(
                            values["Theta"], values["Phi"]);
                }
                else
                {
                    throw AnalyzeError((format("ParsePosition: Unknown 'angle' attribute: %s") % attrs["angle"]).str(), GetLine(reader));
                }
            }
            else
            {
                throw AnalyzeError((format("ParsePosition: Unknown 'type' attribute: %s") % attrs["type"]).str(), GetLine(reader));
            }

        }

        // position
        else
        {
            if (attrs["unit"] != "meter")
            {
                // convert unit to meter

                throw AnalyzeError("ParsePosition: Unknown 'unit' attribute", GetLine(reader));
            }

            if (attrs["type"] == "cartesian")
            {
                return MakePositionFromVector<float>(
                        Vector3<float>(values["X"], values["Y"], values["Z"]));
            }
            else if (attrs["type"] == "unitvector")
            {
                return MakePositionFromVector<float>(
                        Vector3<float>(values["X"], values["Y"], values["Z"]), values["R"]);
            }
            else if (attrs["type"] == "polar")
            {
                if (Contains(attrs, "angle") == false || attrs["angle"] == "degree")
                {
                    return MakePositionFromPolar<float>(
                            DegreeToRadian(values["Theta"]), DegreeToRadian(values["Phi"]), values["R"]);
                }
                else if (attrs["angle"] == "radian")
                {
                    return MakePositionFromPolar<float>(
                            values["Theta"], values["Phi"], values["R"]);
                }
                else
                {
                    throw AnalyzeError((format("ParsePosition: Unknown 'angle' attribute: %s") % attrs["angle"]).str(), GetLine(reader));
                }
            }
            else
            {
                throw AnalyzeError((format("ParsePosition: Unknown 'type' attribute: %s") % attrs["type"]).str(), GetLine(reader));
            }

        }

    }
    else
    {
        throw AnalyzeError("ParsePosition: The given node isn't Position start tag.", GetLine(reader));
    }
}

std::vector<unsigned int> ParseMicIndices(xmlpp::TextReader& reader)
{
    vector<unsigned int> result;

    if (IsElement(reader, "MicIndices"))
    {
        if (reader.move_to_attribute("size"))
        {
            result.reserve(lexical_cast<size_t>(reader.get_value()));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "MicIndices"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                result.push_back(ParseGeneral<unsigned int>(reader, "MicIndex"));
            }
        }
    }
    else
    {
        throw AnalyzeError("ParseMicIndices: The given node isn't MicIndices start tag.", GetLine(reader));
    }

    return result;
}

harkconfig::File1 ParseFile1(xmlpp::TextReader& reader)
{
    File1 result;

    if (IsElement(reader, "File"))
    {
        while (reader.read())
        {
            if (IsEndElement(reader, "File"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "Name")
                {
                    result.SetName(reader.read_inner_xml());
                }
                else if (reader.get_name() == "MicIndices")
                {
                    result.SetMicIndices(ParseMicIndices(reader));
                }
                else if (reader.get_name() == "Comment")
                {
                    result.SetComment(ParseComment(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseFile1: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }

            }
        }
    }
    else
    {
        throw AnalyzeError("ParseFile1: The given node isn't File start tag.", GetLine(reader));
    }

    return result;
}

std::vector<harkconfig::File1> ParseFiles1(xmlpp::TextReader& reader)
{
    vector<File1> result;

    if (IsElement(reader, "Files"))
    {
        if (reader.move_to_attribute("size"))
        {
            result.reserve(lexical_cast<size_t>(reader.get_value()));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "Files"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "File")
                {
                    result.push_back(ParseFile1(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseFiles1: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }
        }
    }
    else
    {
        throw AnalyzeError("ParseFiles1: The given node isn't Files start tag.", GetLine(reader));
    }

    return result;
}

harkconfig::Source2 ParseSource2(xmlpp::TextReader& reader)
{
    Source2 result;

    if (IsElement(reader, "Source"))
    {
        while (reader.read())
        {
            if (IsEndElement(reader, "Source"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "Position")
                {
                    result.SetPosition(ParsePosition(reader));
                }
                else if (reader.get_name() == "Files")
                {
                    result.SetFiles(ParseFiles1(reader));
                }
                else if (reader.get_name() == "CutStart")
                {
                    result.SetCutStart(lexical_cast<unsigned int>(reader.read_inner_xml()));
                }
                else if (reader.get_name() == "CutEnd")
                {
                    result.SetCutEnd(lexical_cast<unsigned int>(reader.read_inner_xml()));
                }
                else if (reader.get_name() == "Comment")
                {
                    result.SetComment(ParseComment(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseSource2: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }

        }
    }
    else
    {
        throw AnalyzeError("ParseSource2: The given node isn't Source start tag.", GetLine(reader));
    }

    return result;
}

std::vector<harkconfig::Source2> ParseSources2(xmlpp::TextReader& reader)
{
    vector<Source2> result;

    if (IsElement(reader, "Sources"))
    {
        if (reader.move_to_attribute("size"))
        {
            result.reserve(lexical_cast<size_t>(reader.get_value()));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "Sources"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "Source")
                {
                    result.push_back(ParseSource2(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseSources2: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }

        }
    }
    else
    {
        throw AnalyzeError("ParseSources2: The given node isn't Sources start tag.", GetLine(reader));
    }

    return result;
}

unsigned int ParseNumMic(xmlpp::TextReader& reader)
{
    return ParseGeneral<unsigned int>(reader, "NumMic");
}

harkio2::FileType::Kind ParseSourceFileType(xmlpp::TextReader& reader)
{
    if (IsElement(reader, "SourceFileType"))
    {
        Glib::ustring type = reader.read_inner_xml();

        harkio2::FileType::Kind typeKind;
        if (ConvertFileType(type, typeKind))
        {
            return typeKind;
        }
        else
        {
            throw AnalyzeError((format("ParseSourceFileType: Unknown file type: %s") % type).str(), GetLine(reader));
        }
    }
    else
    {
        throw AnalyzeError("ParseSourceFileType: The given node isn't ParseSourceFileType start tag.", GetLine(reader));
    }
}

Glib::ustring ParseComment(xmlpp::TextReader& reader)
{
    return ParseGeneral<Glib::ustring>(reader, "Comment");
}

unsigned int ParseSynchronousAverage(xmlpp::TextReader& reader)
{
    return ParseGeneral<unsigned int>(reader, "SynchronousAverage");
}

unsigned int ParseTSPOffset(xmlpp::TextReader& reader)
{
    return ParseGeneral<unsigned int>(reader, "TSPOffset");
}

unsigned int ParseTSPLength(xmlpp::TextReader& reader)
{
    return ParseGeneral<unsigned int>(reader, "TSPLength");
}

unsigned int ParseSignalMax(xmlpp::TextReader& reader)
{
    return ParseGeneral<unsigned int>(reader, "SignalMax");
}

harkio2::TypedFilename ParseOriginalImpulseFile(xmlpp::TextReader& reader)
{
    if (IsElement(reader, "OriginalImpulseFile"))
    {
        Glib::ustring name = reader.read_inner_xml();
        harkio2::FileType::Kind type;

        if (reader.move_to_attribute("type"))
        {
            if (!ConvertFileType(reader.get_value(), type))
            {
                throw AnalyzeError("ParseOriginalImpulseFile: Invalid 'type' attribute.", GetLine(reader));
            }

        }
        else
        {
            throw AnalyzeError("ParseOriginalImpulseFile: The given node doesn't contain a 'type' attribute.", GetLine(reader));
        }

        // move to the end element
        while (reader.read())
        {
            if (IsEndElement(reader, "OriginalImpulseFile"))
            {
                break;
            }
        }

        return harkio2::TypedFilename(name, type);
    }
    else
    {
        throw AnalyzeError("ParseOriginalImpulseFile: The given node isn't OriginalImpulseFile start tag.", GetLine(reader));
    }
}

harkconfig::Mic1 ParseMic1(xmlpp::TextReader& reader)
{
    harkconfig::Mic1 result;

    if (IsElement(reader, "Mic"))
    {
        while (reader.read())
        {
            if (IsEndElement(reader, "Mic"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "MicIndex")
                {
                    result.SetMicIndex(ParseGeneral<unsigned int>(reader, "MicIndex"));
                }
                else if (reader.get_name() == "Position")
                {
                    result.SetPosition(ParsePosition(reader));
                }
                else if (reader.get_name() == "Comment")
                {
                    result.SetComment(ParseComment(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseMic1: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }

            }
        }
    }
    else
    {
        throw AnalyzeError("ParseMic1: The given node isn't Mic start tag.", GetLine(reader));
    }

    return result;
}

std::vector<harkconfig::Mic1> ParseMics1(xmlpp::TextReader& reader)
{
    vector<harkconfig::Mic1> result;

    if (IsElement(reader, "Mics"))
    {
        if (reader.move_to_attribute("size"))
        {
            result.reserve(lexical_cast<size_t>(reader.get_value()));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "Mics"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "Mic")
                {
                    result.push_back(ParseMic1(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseMics1: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }
        }
    }
    else
    {
        throw AnalyzeError("ParseMics1: The given node isn't Mics start tag.", GetLine(reader));
    }

    return result;
}

harkconfig::NoiseSource1 ParseNoiseSource1(xmlpp::TextReader& reader)
{
    harkconfig::NoiseSource1 result;

    if (IsElement(reader, "NoiseSource"))
    {
        while (reader.read())
        {
            if (IsEndElement(reader, "NoiseSource"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "NoiseIndex")
                {
                    result.SetNoiseIndex(ParseGeneral<unsigned int>(reader, "NoiseIndex"));
                }
                else if (reader.get_name() == "Position")
                {
                    result.SetPosition(ParsePosition(reader));
                }
                else if (reader.get_name() == "Comment")
                {
                    result.SetComment(ParseComment(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseNoiseSource1: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }

            }
        }
    }
    else
    {
        throw AnalyzeError("ParseNoiseSource1: The given node isn't NoiseSource start tag.", GetLine(reader));
    }

    return result;
}

std::vector<harkconfig::NoiseSource1> ParseNoiseSources1(xmlpp::TextReader& reader)
{
    vector<harkconfig::NoiseSource1> result;

    if (IsElement(reader, "NoiseSources"))
    {
        if (reader.move_to_attribute("size"))
        {
            result.reserve(lexical_cast<size_t>(reader.get_value()));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "NoiseSources"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "NoiseSource")
                {
                    result.push_back(ParseNoiseSource1(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseNoiseSources1: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }
        }
    }
    else
    {
        throw AnalyzeError("ParseNoiseSources1: The given node isn't NoiseSources start tag.", GetLine(reader));
    }

    return result;
}

boost::shared_ptr<harkconfig::ImpulseResponseListFile> ParseImpulseResponseListFile(xmlpp::TextReader& reader)
{
    if (IsElement(reader, "hark_config"))
    {
        if (reader.get_attribute("id") == "ImpulseResponseList")
        {
            throw AnalyzeError("ParseImpulseResponseListFile: Invalid id", GetLine(reader));
        }

        if (reader.get_attribute("major") == "2" && reader.get_attribute("minor") == "0")
        {
            return static_pointer_cast<harkconfig::ImpulseResponseListFile>(
                    ParseImpulseResponseListFile2_0(reader));
        }
        else
        {
            throw AnalyzeError("ParseImpulseResponseListFile: Invalid version number", GetLine(reader));
        }
    }
    else
    {
        throw AnalyzeError("ParseImpulseResponseListFile: The given node isn't hark_config start tag.", GetLine(reader));
    }
}

boost::shared_ptr<harkconfig::ImpulseResponseListFile2_0> ParseImpulseResponseListFile2_0(xmlpp::TextReader& reader)
{
    boost::shared_ptr<harkconfig::ImpulseResponseListFile2_0> result(
            new harkconfig::ImpulseResponseListFile2_0());

    if (IsElement(reader, "hark_config"))
    {
        if (!(reader.get_attribute("id") == "ImpulseResponseList" &&
                    reader.get_attribute("major") == "2" &&
                    reader.get_attribute("minor") == "0"))
        {
            throw AnalyzeError("ParseImpulseResponseListFile2_0: Invalid attribute(s)", GetLine(reader));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "hark_config"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "NumMic")
                {
                    result->SetNumMic(ParseNumMic(reader));
                }
                else if (reader.get_name() == "SourceFileType")
                {
                    result->SetSourceFileType(ParseSourceFileType(reader));
                }
                else if (reader.get_name() == "Comment")
                {
                    result->SetComment(ParseComment(reader));
                }
                else if (reader.get_name() == "Sources")
                {
                    result->SetSources(ParseSources2(reader));

                    // Check NumMic consistency.
                    for (size_t i = 0; i < result->GetSources().size(); ++i)
                    {
                        const harkconfig::Source2& source = result->GetSources()[i];

                        // Calculate total number of MicIndices.size.
                        size_t n = 0;
                        for (size_t j = 0; j < source.GetFiles().size(); ++j)
                        {
                            const harkconfig::File1& file = source.GetFiles()[j];
                            n += file.GetMicIndices().size();
                        }

                        if (result->IsNumMicInitialized())
                        {
                            if (result->GetNumMic() != n)
                            {
                                throw AnalyzeError("ParseImpulseResponseListFile2_0: Invalid NumMic", GetLine(reader));
                            }
                        }
                        else
                        {
                            // First setting.
                            result->SetNumMic(n);
                        }
                    }
                }
                else
                {
                    throw AnalyzeError((format("ParseImpulseResponseListFile2_0: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }

        }
    }
    else
    {
        throw AnalyzeError("ParseImpulseResponseListFile2_0: The given node isn't hark_config start tag.", GetLine(reader));
    }

    return result;
}

boost::shared_ptr<harkconfig::TSPListFile> ParseTSPListFile(xmlpp::TextReader& reader)
{
    if (IsElement(reader, "hark_config"))
    {
        if (reader.get_attribute("id") == "TSPList")
        {
            throw AnalyzeError("ParseTSPListFile: Invalid id", GetLine(reader));
        }

        if (reader.get_attribute("major") == "2" && reader.get_attribute("minor") == "0")
        {
            return static_pointer_cast<harkconfig::TSPListFile>(
                    ParseTSPListFile2_0(reader));
        }
        else
        {
            throw AnalyzeError("ParseTSPListFile: Invalid version number", GetLine(reader));
        }
    }
    else
    {
        throw AnalyzeError("ParseTSPListFile: The given node isn't hark_config start tag.", GetLine(reader));
    }
}

boost::shared_ptr<harkconfig::TSPListFile2_0> ParseTSPListFile2_0(xmlpp::TextReader& reader)
{
    boost::shared_ptr<harkconfig::TSPListFile2_0> result(
            new harkconfig::TSPListFile2_0());

    if (IsElement(reader, "hark_config"))
    {
        if (!(reader.get_attribute("id") == "TSPList" &&
                    reader.get_attribute("major") == "2" &&
                    reader.get_attribute("minor") == "0"))
        {
            throw AnalyzeError("ParseTSPListFile2_0: Invalid attribute(s)", GetLine(reader));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "hark_config"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "NumMic")
                {
                    result->SetNumMic(ParseNumMic(reader));
                }
                else if (reader.get_name() == "SourceFileType")
                {
                    result->SetSourceFileType(ParseSourceFileType(reader));
                }
                else if (reader.get_name() == "Comment")
                {
                    result->SetComment(ParseComment(reader));
                }
                else if (reader.get_name() == "SynchronousAverage")
                {
                    result->SetSynchronousAverage(ParseSynchronousAverage(reader));
                }
                else if (reader.get_name() == "OriginalImpulseFile")
                {
                    result->SetOriginalImpulseFile(ParseOriginalImpulseFile(reader));
                }
                else if (reader.get_name() == "TSPOffset")
                {
                    result->SetTSPOffset(ParseTSPOffset(reader));
                }
                else if (reader.get_name() == "TSPLength")
                {
                    result->SetTSPLength(ParseTSPLength(reader));
                }
                else if (reader.get_name() == "SignalMax")
                {
                    result->SetSignalMax(ParseSignalMax(reader));
                }
                else if (reader.get_name() == "Sources")
                {
                    result->SetSources(ParseSources2(reader));

                    // Check NumMic consistency.
                    for (size_t i = 0; i < result->GetSources().size(); ++i)
                    {
                        const harkconfig::Source2& source = result->GetSources()[i];

                        // Calculate total number of MicIndices.size.
                        size_t n = 0;
                        for (size_t j = 0; j < source.GetFiles().size(); ++j)
                        {
                            const harkconfig::File1& file = source.GetFiles()[j];
                            n += file.GetMicIndices().size();
                        }

                        if (result->IsNumMicInitialized())
                        {
                            if (result->GetNumMic() != n)
                            {
                                throw AnalyzeError("ParseTSPListFile2_0: Invalid NumMic", GetLine(reader));
                            }
                        }
                        else
                        {
                            // First setting.
                            result->SetNumMic(n);
                        }
                    }
                }
                else
                {
                    throw AnalyzeError((format("ParseTSPListFile2_0: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }

        }
    }
    else
    {
        throw AnalyzeError("ParseTSPListFile2_0: The given node isn't hark_config start tag.", GetLine(reader));
    }

    return result;
}

boost::shared_ptr<harkconfig::MicArrayLocationFile> ParseMicArrayLocationFile(xmlpp::TextReader& reader)
{
    if (IsElement(reader, "hark_config"))
    {
        if (reader.get_attribute("id") == "MicArrayLocation")
        {
            throw AnalyzeError("ParseMicArrayLocationFile: Invalid id", GetLine(reader));
        }

        if (reader.get_attribute("major") == "2" && reader.get_attribute("minor") == "0")
        {
            return static_pointer_cast<harkconfig::MicArrayLocationFile>(
                    ParseMicArrayLocationFile2_0(reader));
        }
        else
        {
            throw AnalyzeError("ParseMicArrayLocationFile: Invalid version number", GetLine(reader));
        }
    }
    else
    {
        throw AnalyzeError("ParseMicArrayLocationFile: The given node isn't hark_config start tag.", GetLine(reader));
    }
}

boost::shared_ptr<harkconfig::MicArrayLocationFile2_0> ParseMicArrayLocationFile2_0(xmlpp::TextReader& reader)
{
    boost::shared_ptr<harkconfig::MicArrayLocationFile2_0> result(
            new harkconfig::MicArrayLocationFile2_0());

    if (IsElement(reader, "hark_config"))
    {
        if (!(reader.get_attribute("id") == "MicArrayLocation" &&
                    reader.get_attribute("major") == "2" &&
                    reader.get_attribute("minor") == "0"))
        {
            throw AnalyzeError("ParseMicArrayLocationFile2_0: Invalid attribute(s)", GetLine(reader));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "hark_config"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "Comment")
                {
                    result->SetComment(ParseComment(reader));
                }
                else if (reader.get_name() == "Mics")
                {
                    result->SetMics(ParseMics1(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseMicArrayLocationFile2_0: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }

        }
    }
    else
    {
        throw AnalyzeError("ParseMicArrayLocationFile2_0: The given node isn't hark_config start tag.", GetLine(reader));
    }

    return result;
}

boost::shared_ptr<harkconfig::NoiseLocationFile> ParseNoiseLocationFile(xmlpp::TextReader& reader)
{
    if (IsElement(reader, "hark_config"))
    {
        if (reader.get_attribute("id") == "NoiseLocation")
        {
            throw AnalyzeError("ParseNoiseLocationFile: Invalid id", GetLine(reader));
        }

        if (reader.get_attribute("major") == "2" && reader.get_attribute("minor") == "0")
        {
            return static_pointer_cast<harkconfig::NoiseLocationFile>(
                    ParseNoiseLocationFile2_0(reader));
        }
        else
        {
            throw AnalyzeError("ParseNoiseLocationFile: Invalid version number", GetLine(reader));
        }
    }
    else
    {
        throw AnalyzeError("ParseNoiseLocationFile: The given node isn't hark_config start tag.", GetLine(reader));
    }
}

boost::shared_ptr<harkconfig::NoiseLocationFile2_0> ParseNoiseLocationFile2_0(xmlpp::TextReader& reader)
{
    boost::shared_ptr<harkconfig::NoiseLocationFile2_0> result(
            new harkconfig::NoiseLocationFile2_0());

    if (IsElement(reader, "hark_config"))
    {
        if (!(reader.get_attribute("id") == "NoiseLocation" &&
                    reader.get_attribute("major") == "2" &&
                    reader.get_attribute("minor") == "0"))
        {
            throw AnalyzeError("ParseNoiseLocationFile2_0: Invalid attribute(s)", GetLine(reader));
        }

        while (reader.read())
        {
            if (IsEndElement(reader, "hark_config"))
            {
                break;
            }
            else if (reader.get_node_type() == TextReader::Element)
            {
                if (reader.get_name() == "Comment")
                {
                    result->SetComment(ParseComment(reader));
                }
                else if (reader.get_name() == "NoiseSources")
                {
                    result->SetNoiseSources(ParseNoiseSources1(reader));
                }
                else
                {
                    throw AnalyzeError((format("ParseNoiseLocationFile2_0: Unknown content: %s") % reader.get_name()).str(), GetLine(reader));
                }
            }

        }
    }
    else
    {
        throw AnalyzeError("ParseNoiseLocationFile2_0: The given node isn't hark_config start tag.", GetLine(reader));
    }

    return result;
}

} // harkconfig
} // harkio

